
SUBROUTINE proc_make_ideal

   USE constants
   USE gridded_data
   USE header_data
   USE namelist_info
   USE terrain_data
   
   IMPLICIT NONE

   INTEGER II, JJ, KK, kxf
   REAL X0, Y0

! scalar functions called
!   REAL HGEOPOT, T2
!   INTEGER find_ter_index

   INTEGER, DIMENSION(nupper_vars) :: idx3d
   INTEGER, DIMENSION(nsfc_vars)   :: idx2d
   INTEGER, DIMENSION(nter_vars)   :: idxTer

   REAL, ALLOCATABLE:: scr2d(:,:), scr3d(:,:,:), ter(:,:)
   REAL, ALLOCATABLE:: T0(:,:), Tv0(:,:), dT_dz(:,:), rho0(:,:), H2P(:,:)
   REAL, ALLOCATABLE:: HP(:,:,:), TP(:,:,:), TPV (:,:,:)
! ASSELA
  REAL, ALLOCATABLE:: RH(:,:,:)
  REAL,	ALLOCATABLE:: rec2vert_temp(:)
! END ASSELA

   INTEGER idom, iexp, ioffst, joffst, ixmunxp, jxmunxp
   REAL dxmoad, xwest, xsouth, xeast, xnorth, x0moad, y0moad
   REAL, ALLOCATABLE :: tmp(:,:,:)

   kxf = SIZE ( all_3d , DIM=3 )

!   print *,'ixf, jxf, kxf',ixf,jxf,kxf,nsfc_vars, nupper_vars
   


   CALL find_2d_indices(idx2d)   ! [1,2,3] = Pseasfc,Tseasfc , Coriolis(check), all_2d
   idx3d = find_3d_indices()   ! [1,2,3,4,5] = T,U,V,RH,HGT, all_3d
! ASSELA : ***3
   CALL find_ter_index(idxTer)    ! index into ter_data
   print *,'Debugging find_ter_index, after call idxTer = ',idxTer
   print *,'Size of ter array: ',size(ter_data(idxTer(1))%array,1), &
	                     size(ter_data(idxTer(1))%array,2)
   print *,'Size of coriolis array: ',size(ter_data(idxTer(2))%array,1), &
	                     size(ter_data(idxTer(2))%array,2)

!  do some sanity testing on indices, pressure levels etc
!   IF (ANY(rec2%P_levels(1:kxf) .NE. pressure(1:kxf))) THEN
!      PRINT '(A)','Namelist P levels not same as models P levels'
!      STOP 'Problems with P_levels_in_Pa'
!   ENDIF

   IF (ANY(idx2d .LT. 1)) THEN
      PRINT '(A)','Didn\'t find some of the 2d data needed in make_ideal'
      STOP 'Missing_2d_data_for_make_ideal'
   ENDIF
   IF (ANY(idx3d .LT. 1)) THEN
      PRINT '(A)','Didn\'t find some of the 3d data needed in make_ideal'
      STOP 'Missing_3d_data_for_make_ideal'
   ENDIF
   IF (ANY(idxTer .LT. 1)) THEN
      PRINT '(A)','Didn\'t find some of the terrain data needed in make_ideal'
      STOP 'Missing_terrain_data_for_make_ideal'
   ENDIF


   idom   = 1                   !  could be used to loop over domains
   dxmoad = 1.0 * bhr(1,1)   !  coarse domain grid distance in m
   iexp   = bhi(8,1)            !  is coarse domain expanded (1) or not (0)
   ioffst = bhi(10,1)           !  follow convention in proc_read_terrain (swap i<->j)
   joffst = bhi(9,1)
   xwest  = bhr(11,1)
   xsouth = bhr(10,1)
   xnorth = bhr(13,1)
   xeast  = bhr(12,1)
   ixmunxp = bhi(6,1)
   jxmunxp = bhi(5,1)

! NOTATION: 
! X0,Y0 : center of the grid. 

!     --- coordinates of (1,1) in MOAD not expanded, dot point
   x0moad = -dxmoad * REAL(ixmunxp-1) * 0.5 
   y0moad = -dxmoad * REAL(jxmunxp-1) * 0.5 
!     --- coordinates of (1,1) in current domain, dot point
   if (idom .eq. 1) then
      x0 = x0moad
      y0 = y0moad
      if (iexp .eq. 1) then
         x0 = x0 - dxmoad*real(ioffst)
         y0 = y0 - dxmoad*real(joffst)
      endif
   else
      x0 = x0moad + dxmoad * (xsouth-1.)
      y0 = y0moad + dxmoad * (xwest -1.)
   endif

   print *,'Pressure: ',pressure
   print *,'P_levels: ',rec2%P_levels(1:kxf)
   print *,'T_levels: ',rec2%T_levels(1:kxf)
   print *,'U_levels: ',rec2%U_levels(1:kxf)
   print *,'V_levels: ',rec2%V_levels(1:kxf)
   print *,'humid: ', rec2%humid(1:kxf)

   print *,'x0,y0:',x0,y0

!  move test down here for debugging purposes
   IF (ANY(rec2%P_levels(1:kxf) .NE. pressure(1:kxf))) THEN
      PRINT '(A)','Namelist P levels not same as models P levels'
! This is really not a problem since we are generating the data anyway
!      STOP 'Problems with P_levels_in_Pa'
   ENDIF

   ALLOCATE(scr2d(jxf,ixf), scr3d(jxf,ixf,kxf), ter(jxf,ixf))
   ALLOCATE(T0(jxf,ixf), Tv0(jxf,ixf), dT_dz(jxf,ixf), TP(jxf,ixf,kxf),TPv(jxf,ixf,kxf), HP(jxf,ixf,kxf))
! ASSELA
  ALLOCATE(RH(jxf,ixf,kxf))
  ALLOCATE(rec2vert_temp(kxf))
  ALLOCATE(tmp(jxf,ixf,kxf))
! END ASSELA

   ter = ter_data(idxTer(1))%array

   all_3d(:,:,:,idx3d(2)) = spread(spread(rec2%U_levels,dim=1,ncopies=jxf),dim=2,ncopies=ixf)
   all_3d(:,:,:,idx3d(3)) = spread(spread(rec2%V_levels,dim=1,ncopies=jxf),dim=2,ncopies=ixf)
   
!   all_2d(:,:,idxTer(2))=1.E-4   !GNP -> problems with Coriolis force
!   scr2d = C_H0(x0,y0,all_3d(:,:,1,idx3d(2)),all_3d(:,:,1,idx3d(3)),all_2d(:,:,idxTer(2)))
   scr2d = C_H0(x0,y0,all_3d(:,:,1,idx3d(2)),all_3d(:,:,1,idx3d(3)),ter_data(idxTer(2))%array)

!  Temperature, T = T_0 + z*(dT/dz)_0
!  Need to double-check the formulas, following is just imported from idealv2

! if first temperature is set at a large negative value, that means we need to read it from 
! pregrid output
   IF(rec2%T_levels(1)< -273) THEN 
	WRITE (*,*) 'Reading surface temperature from pregrid output (T_levels(1)<-273'
	T0=all_3d(:,:,1,idx3d(1))
   ELSE
	WRITE(*,*) 'Using T_levels(1) as surface temperature'
	T0=rec2%T_levels(1)
   END IF
   T0 = 300 !all_3d(:,:,1,idx3d(1))    !  or use SST = all_2d(:,:,idx2d(1))?
   HP(:,:,1) = 0.
   TP(:,:,1) = T0
   HP(:,:,2) = 0.
   TP(:,:,2) = T0
! ASSELA
!  print 'ASSELA : humid ', rec2%humid(1:kxf)
  RH(:,:,1)=rec2%humid(1)
  RH(:,:,2)=rec2%humid(2)
! END ASSELAx-section.input.t2.1.log
!WRITE(59,*)"P1\t\tT1\t\tHP1\t\tT\t\tP2\t\tT2\t\tHP2"

! Compute virtual temperatue for each level 
DO KK=1, kxf
	rec2vert_temp(KK)=virtual_temperature(rec2%humid(KK), rec2%T_levels(KK), rec2%P_levels(KK))
END DO
  KK: DO KK=1,2
  II:    DO II=1,ixf
  JJ:      DO JJ=1,jxf
		TPv(JJ,II,KK)=rec2vert_temp(KK)
	   END DO JJ
	 END DO II
      END DO KK

   DO KK=3,kxf
      DO II=1,ixf
         DO JJ=1,jxf
	    RH(JJ,II,KK) = rec2%humid(KK)
            TP(JJ,II,KK) = rec2%T_levels(KK)
            TPv(JJ,II,KK)=rec2vert_temp(KK)
!	    IF(II==10 .AND. JJ==10) THEN
!		WRITE(58,423) TP(JJ,II,KK),saturation_vp(TP(JJ,II,KK)),virtual_temperature(RH(JJ,II,KK), TP(JJ,II,KK), rec2%P_levels(KK))
!	    END IF
            HP(JJ,II,KK) = geopot (HP(JJ,II,KK-1), TPv(JJ,II,KK-1), TPv(JJ,II,KK), &
			       		rec2%P_levels(KK-1), rec2%P_levels(KK))
	    tmp(JJ,II,KK)=geopot (tmp(JJ,II,KK-1), TP(JJ,II,KK-1), TP(JJ,II,KK), &
					rec2%P_levels(KK-1), rec2%P_levels(KK))
!geopot(geopot1, Tv1, Tv2, P1, P2)
	    IF(II==10 .AND. JJ==10) THEN
		    WRITE(59,423)rec2%P_levels(KK),TP(JJ,II,KK),TPv(JJ,II,KK),&
				 TP(JJ,II,KK),TPv(JJ,II,KK), &
				 tmp(JJ,II,KK),HP(jj,ii,kk), HP(jj,ii,kk)-tmp(JJ,II,KK)
		    
	    END IF
         ENDDO
      ENDDO
   ENDDO
423 FORMAT(10(E12.6,"\t"))
!   print *,'T0',maxval(T0),minval(T0)
   dT_dz(:,:) = (rec2%T_levels(3)-rec2%T_levels(2))/ &
		layer_thickness(rec2vert_temp(2),rec2vert_temp(3),&
			        rec2%P_levels(2),rec2%P_levels(3))
!T0*rec2%N2_levels(1)/PC_G - PC_KAPPA*PC_G/PC_R
   

!   print *,'dT_dz:',maxval(dT_dz),minval(dT_dz)

   TP(:,:,1) = TP(:,:,1) + dT_dz*ter

!	WRITE(59,234)"LT",layer_thickness(rec2%T_levels(2),rec2%T_levels(3),&
!			        rec2%P_levels(2),rec2%P_x-section.input.t2.1.loglevels(3))
!	WRITE(59,234)"dT",(rec2%T_levels(3)-rec2%T_levels(2))
!	WRITE(59,234)"TP",TP(11,:,1)
!	WRITE(59,234)"dT_dz", dT_dz(11,:)
!	WRITE(59,234)"TER",ter(11,:)
!234   FORMAT (A,"\t",500(E10.3,"\t"))

   all_3d(:,:,:,idx3d(1)) = TP
! ASSELA
   all_3d(:,:,:,idx3d(4)) = RH
! ASSELA :see /
! END ASSELA

   print *,'TP:',maxval(TP),minval(TP)
   print *,'RH: ',maxval(RH),minval(RH)

! calculate virtual temperature at the surface

Tv0(1:jxf,1:ixf)=virtual_temperature(&
		interpolate(HP(1:jxf,1:ixf,:),RH(1:jxf,1:ixf,:),ter(1:jxf,1:ixf),kxf), &
		interpolate(HP(1:jxf,1:ixf,:),rec2%T_levels(:),ter(1:jxf,1:ixf),kxf), &
		interpolate(HP(1:jxf,1:ixf,:),rec2%P_levels(:),ter(1:jxf,1:ixf),kxf))
 
!WRITE(58,23433) Tv0(11:11,1:ixf)
!WRITE(58,23433) T0(11:11,1:ixf)
!23433 FORMAT (500(E10.3))
!     --- calculate and write sea-level pressure dot and cross
!     p(h=0) = p(level 2) - Dp/Dz*h(level2)
!     -------------------------------------
   ALLOCATE(rho0(jxf,ixf),H2P(jxf,ixf))
   rho0 = rec2%P_levels(2)/(PC_R*Tv0)
   H2P  = PC_G*RHO0   ! -Dp/Dz
   all_2d(:,:,idx2d(1)) = rec2%P_levels(2) + H2P*scr2d
   all_2d(:,:,idx2d(2)) = T0 !Assela added this line. purpose: to make surface temperature
			     !equal to the surface temp. from 3d field. 
   all_3d(:,:,1,idx3d(5)) = ter

   DO KK=2,kxf
      all_3d(:,:,KK,idx3d(5)) = scr2d+HP(:,:,KK)
   ENDDO
   
   DEALLOCATE(scr2d, T0, dT_dz, TP, HP, rho0, H2P, ter)


CONTAINS

   FUNCTION find_3d_indices
!  ----------------------------------------------------------------
!   
!  ----------------------------------------------------------------

   USE constants
   USE gridded_data
   
   IMPLICIT NONE

   INTEGER, DIMENSION(nupper_vars) :: find_3d_indices
   INTEGER loop

   find_3d_indices = -1             ! if not found

   find_names_3d : DO loop = 1 , fg_variables_up_index
      IF      ( all_names_up(loop)(1:3) .EQ. 'T  ' ) THEN
         find_3d_indices(1) = loop
      ELSE IF ( all_names_up(loop)(1:3) .EQ. 'U  ' ) THEN
         find_3d_indices(2) = loop
      ELSE IF ( all_names_up(loop)(1:3) .EQ. 'V  ' ) THEN
         find_3d_indices(3) = loop
      ELSE IF ( all_names_up(loop)(1:3) .EQ. 'RH ' ) THEN
         find_3d_indices(4) = loop
      ELSE IF ( all_names_up(loop)(1:3) .EQ. 'HGT' ) THEN
         find_3d_indices(5) = loop
      END IF
   END DO find_names_3d

   END FUNCTION find_3d_indices
!  ----------------------------------------------------------------


   SUBROUTINE find_2d_indices(idx)
!  ----------------------------------------------------------------
!  For some unknown reason, pgf90 doesn't like this to be a function
!  a'la find_3d_indices, so we workaround by using a subroutine 
!  ----------------------------------------------------------------
   USE constants
   USE gridded_data
   
   IMPLICIT NONE

   INTEGER, DIMENSION(nsfc_vars) :: idx
   INTEGER loop

   idx = -1             ! if not found

   find_names_2d : DO loop = 1 , fg_variables_sfc_index

!      print *,loop,all_names_sfc(loop)(1:8)

      IF      ( ( all_names_sfc(loop)(1:8) .EQ. 'PMSL    ' ) .OR. &
                ( all_names_sfc(loop)(1:8) .EQ. 'PRESSURE' ) .OR. &
                ( all_names_sfc(loop)(1:8) .EQ. 'PSEALVLC' )) THEN
         idx(1) = loop
      ELSE IF ( ( all_names_sfc(loop)(1:8) .EQ. 'SST     ' ) .OR. &
                ( all_names_sfc(loop)(1:8) .EQ. 'SKINTEMP' ) .OR. &
                ( all_names_sfc(loop)(1:8) .EQ. 'TSEASFC ' )) THEN
         idx(2) = loop
! Coriolis force has not been copied to all_2d array, so the following
! test does not find it there. Instead we look in the ter_data data
! structure, where it actually is (see find_ter_index)
!      ELSE IF (   all_names_sfc(loop)(1:8) .EQ. 'CORIOLIS'  ) THEN
!         idx(3) = loop
      END IF
   END DO find_names_2d

!   print *,'Debugging find_2d_indices: idx = ',idx
   END SUBROUTINE find_2d_indices
!  ----------------------------------------------------------------

  
   SUBROUTINE find_ter_index(idx)
!  ----------------------------------------------------------------
!   
!  ----------------------------------------------------------------
   USE constants
   USE gridded_data
   USE terrain_data
   
   IMPLICIT NONE

   INTEGER idx(nter_vars), loop

   idx = -1             ! if not found
   find_names_ter : DO loop = 1 , SIZE(ter_data)
!      print *,'Debugging find_ter_index loop: ',loop,ter_data(loop)%name_info(1:8)
      IF      ( ter_data(loop)%name_info(1:8) .EQ. 'TERRAIN ' ) THEN
         idx(1) = loop
      ELSE IF ( ter_data(loop)%name_info(1:8) .EQ. 'CORIOLIS'  ) THEN
         idx(2) = loop
      END IF
   END DO find_names_ter
!   print *,'Debugging find_ter_index idx: ',idx

   END SUBROUTINE find_ter_index
!  ----------------------------------------------------------------


!   REAL FUNCTION   T2( P1,T1,N2M,P2 )
!!  ----------------------------------------------------------------
!!     calculate temperature T2 in K from pressure p1 (hPa) and
!!     pressure p2 (hPa). Assuming that the temperature at level 1
!!     is T1 (K) and the Brunt Vaisalla frequency between level1
!!     and level 2 is constant n2m (1/s)
!!  NB only depends on P1/P2 so P1 and P2 just need to be in same units
!!  ----------------------------------------------------------------
!
!   USE constants
! 
!   IMPLICIT NONE
!
!   REAL P1, T1, N2M, P2, B
!
!   B = PC_R*N2M/(PC_G**2 * PC_KAPPA)
!   T2=1.0/(B+( -B + 1.0/T1)*((P1/P2)**PC_KAPPA))
!   END FUNCTION T2
!  ----------------------------------------------------------------


!   REAL FUNCTION HGEOPOT(P1,T1,HGEOPOT1,N2M,P2)
!!  ----------------------------------------------------------------
!!     calculate geopot. height in m at level 2 from pressure p1 (hPa)
!!     and pressure p2 (hPa). Assuming that the geopot. height at
!!     level 1 is hgeopot1 (K) and T = T1
!!     and the Brunt Vaisalla frequency between
!!     level1 and level 2 is constant n2m (1/s)
!!  NB only depends on P2/P1 so P1 and P2 just need to be in same units
!!  ----------------------------------------------------------------
!   USE constants
!
!   IMPLICIT NONE
!
!   REAL P1, HGEOPOT1, N2M, P2, T1
!   REAL B
!
!   B = PC_R*N2M/(PC_G**2 * PC_KAPPA)
!
!   HGEOPOT =  HGEOPOT1 - PC_G/N2M * LOG(B*T1*(P2/P1)**PC_KAPPA+1.0-B*T1)
!   END FUNCTION HGEOPOT
!  ----------------------------------------------------------------


   REAL FUNCTION H0(I, J, POINT, X0, Y0, Uij, Vij, Fij)
!  ----------------------------------------------------------------
!     calculate the geopotential height (m) of the lowest pressure
!     surface ; input are grid indices i (S-N) and j (W-E).
!     (x0,y0) is the location in m of the point (1,1) dot in the
!     current domain.
!  ----------------------------------------------------------------
   USE constants
   USE header_data

   IMPLICIT NONE

   INTEGER I, J, POINT
   REAL X, Y, X0, Y0, Uij, Vij, Fij

!  Fij = coriolis, Uij & Vij velocity components on surface

   X=1000.*grid_distance_km*(REAL(J-1)+0.5*REAL(POINT)) + X0
   Y=1000.*grid_distance_km*(REAL(I-1)+0.5*REAL(POINT)) + Y0
   H0 = Fij/PC_G*(Vij*X-Uij*Y)

!      WRITE(58,423)X,Y,Vij,Uij,PC_G,Fij,H0
!`423 FORMAT("Computing with X,Y,V,U,G,F (To get H) ="10(F12.3,"\t"))

   END FUNCTION H0
!  ----------------------------------------------------------------


   FUNCTION C_H0(X0,Y0,U,V,F)
!  ----------------------------------------------------------------
!     calculate h0( i,j) [Calculates geopotential height(m) of lowest pressure surface]
!     for a field of size ix,jx, on dot points.
!     (x0,y0) is the location in m of the point (1,1) dot in the
!     current domain.
!  ----------------------------------------------------------------
   USE constants
   USE header_data

   IMPLICIT NONE

!  ixf, jxf come from header_data
   REAL, DIMENSION(jxf,ixf) :: C_H0, U, V, F
   REAL H0, X0, Y0
   INTEGER II, JJ

   DO II=1,ixf
      DO JJ=1,jxf
         C_H0(JJ,II)=H0(JJ,II,0,X0,Y0, &
              U(JJ,II),V(JJ,II), &
              F(JJ,II))
      ENDDO
   ENDDO
   END FUNCTION C_H0
!  ----------------------------------------------------------------

!REAL FUNCTION geopot (geopot1, Tv1, Tv2, P1, P2)
!! Computes GEOPOTENTIAL HEIGHT using Z2=Z1+Rd/g(Tv2+Tv1)/2.ln(P1/P2)
!! for dry cases, Tv=T
!! (Tv2+Tv1)/2.0 is approx. Works only for thin slices
!! page 55 of Wallace and Hobbs
!USE constants
!IMPLICIT NONE
!	REAL :: Tv1, Tv2, P1, P2, geopot1
!	geopot=geopot1+PC_R/PC_G*(Tv2+Tv1)/2.*LOG(P1/P2)
!END FUNCTION geopot
!
!! END MODULE make_ideal
!! ----------------------------------------------------------------------

REAL FUNCTION geopot (geopot1, Tv1, Tv2, P1, P2)
	REAL :: Tv1, Tv2, P1, P2, geopot1
	geopot=geopot1+layer_thickness(Tv1,Tv2,P1,P2)
END FUNCTION geopot

REAL FUNCTION layer_thickness(Tv1, Tv2, P1, P2)
! compultes thickness of airlayer using
! Non-isothermal :Z2-Z1=(Tv2-Tv1)*LOG(P1/P2)/LOG(Tv2/Tv1)*PC_R/PC_G
! Isothermal : Z2-Z1=PC_R/PC_G*(Tv1+Tv2)/2.*LOG(P1/P2)
! See proof on back page of wallace and hobbs copy 
! based on page 60
USE constants
IMPLICIT NONE
	
	REAL :: Tv1, Tv2, P1, P2

!	WRITE(*,*) "HERE:"
!	WRITE(*,50)  Tv1, Tv2, P1, P2, LOG(P1/P2), PC_R/PC_G ,LOG(Tv2/Tv1)

!	WRITE(*,*) ":END"
	IF ( Tv1<>Tv2) THEN
		layer_thickness=(Tv2-Tv1)*LOG(P1/P2)*PC_R/PC_G*1/LOG(Tv2/Tv1)
	ELSE
		layer_thickness=PC_R/PC_G*(Tv1+Tv2)/2.*LOG(P1/P2)
	END IF
END FUNCTION layer_thickness

! ----------------------------------------------------------------------

REAL FUNCTION saturation_vp(T)
! computes saturation vapor pressure based on 6th order polynormial method
! proposed by # 6th order Polynomial (Flatau et. al.,1992) (-50 to 50C)
! Journal of Applied Meteorology: Vol. 31, No. 12, pp. 1507?1513.
!  OUTPUT: hPa (mb)
REAL :: T ! temperature in K
REAL :: tc
REAL :: para(7)=(/6.11176750 ,0.443986062 ,0.143053301E-01 ,0.265027242E-03 , 0.302246994E-05 , 0.203886313E-07 , 0.638780966E-10/)
INTEGER :: i
tc=T-273.15
saturation_vp=para(1)
DO i=2, 7
saturation_vp=saturation_vp+para(i)*(tc**(i-1))
!write(58,*)saturation_vp,para(i),tc**(i-1)
END DO


END FUNCTION saturation_vp
! _------------------------------------------------------------------------

REAL FUNCTION virtual_temperature(RH, T, P)
REAL :: RH, T, P ! %, Kelvin, Pa (1/100 mb)
REAL :: vp
vp=RH/100.*saturation_vp(T)*100.
virtual_temperature=T/(1-vp/P*(1-.622))
! WRITE(58,*) "Calling with RH, T, P gave vp, vt", RH, T, P, vp, virtual_temperature

END FUNCTION virtual_temperature
! _------------------------------------------------------------------------

REAL FUNCTION interpolate(xset, yset, xval, n)
! searches XSET for envilop of XVAL and interpolate linearly to find corressponding
! YVAL
! Expects the xset to be in acsending order. 
INTEGER :: n
REAL, DIMENSION(n):: xset, yset
REAL :: xval
integer :: ll, uu
INTEGER :: i
IF (xval <= xset(1)) THEN 
	interpolate=yset(1)
ELSEIF (xval >= xset(n)) THEN 
	interpolate=yset(n)
ELSE
		! xval is in the range
	DO i=1, n 
		IF(xset(i) <= xval) THEN
			ll=i
		END IF
	END DO
	DO i=n, 1, -1
		IF(xset(i) >=xval) THEN 
			uu=i
		END IF
	END DO
	! now interpolate
	IF (xset(uu)-xset(ll)==0) THEN
		interpolate=.5*(yset(uu)+yset(ll))
	ELSE
		interpolate=yset(ll)+(xval-xset(ll))*(yset(uu)-yset(ll))/(xset(uu)-xset(ll))
	END IF

END IF
END FUNCTION interpolate
!-----------------------------------------------------------------------------
END SUBROUTINE proc_make_ideal








