SUBROUTINE proc_namelist

   USE date_pack
   USE namelist_info
   USE file_data

   IMPLICIT NONE

   INTEGER                :: start_year       , &
                             start_month      , &
                             start_day        , &
                             start_hour       , &
                             start_minute = 0 , &
                             start_second = 0
   INTEGER                :: end_year         , &
                             end_month        , &
                             end_day          , &
                             end_hour         , &
                             end_minute   = 0 , &
                             end_second   = 0
   INTEGER                :: interval
   REAL                   :: ptop_in_Pa , sst_to_ice_threshold
   INTEGER, PARAMETER     :: end_marker = -9999
   REAL, DIMENSION(max_P_levels) :: P_levels_in_Pa = end_marker, T_levels = end_marker, &
                                    U_levels = end_marker, V_levels = end_marker, humid = end_marker
   LOGICAL                :: linear_interpolation
   INTEGER                :: i , j
   CHARACTER(LEN=100),DIMENSION(100) :: root                = (/ ( "                    ",I=1,100 ) /)
   CHARACTER(LEN=100),DIMENSION(100) :: constants_full_name = (/ ( "                    ",I=1,100 ) /)
   CHARACTER(LEN=100)                :: terrain_file_name   =      "                    " 
   LOGICAL                :: print_echo , print_debug , print_mask , &
                             print_interp , print_link_list_store , &
                             print_array_store , print_header , print_output , &
                             print_file , print_f77_info

   NAMELIST / record1 / start_year   , start_month  , start_day    , &
                        start_hour   , start_minute , start_second , &
                        end_year     , end_month    , end_day      , &
                        end_hour     , end_minute   , end_second   , &
                        interval
   NAMELIST / record2 / ptop_in_Pa , P_levels_in_Pa , T_levels, U_levels, V_levels, humid, &
                        sst_to_ice_threshold , linear_interpolation
   NAMELIST / record3 / root , constants_full_name , terrain_file_name
   NAMELIST / record4 / print_echo , print_debug , print_mask , &
                        print_interp , print_link_list_store , &
                        print_array_store , print_header , print_output , &
                        print_file , print_f77_info

   !  This data is the local stuff for the time loop.  We
   !  compute the incremental times from the starting and
   !  ending times defined in the NAMELIST. 

   INTEGER :: cy1,mm1,dd1,hh1,mi1,ss1
   INTEGER :: cy2,mm2,dd2,hh2,mi2,ss2
   CHARACTER (LEN=19) :: current_date , next_date

   !  To compute how many new levels are to be added to the analysis
   !  through vertical interpolation, we need an index.

   INTEGER :: levels_index, number_of_P_levels

   !  Find a spare UNIT number for the NAMELIST file.

   namelist_file = -1
   namelist_unit : DO file_index = file_index_min , file_index_max
      INQUIRE (UNIT=file_index, OPENED=already_opened)
      IF ( .NOT. already_opened) THEN
         namelist_file = file_index
         EXIT namelist_unit
      END IF
   END DO namelist_unit   
   
   IF ( namelist_file .EQ. -1 ) THEN
      PRINT '(A)','Could not find a spare unit for the NAMELIST data.'
      STOP 'namelist_file_unit'
   END IF

   !  Play it safe and REWIND the file.

   REWIND ( UNIT=namelist_file )

   PRINT '(A,A/)','Trying to open file: ',file_name_namelist

   !  OPEN the NAMELIST file with the given name and UNIT.

   OPEN ( UNIT=namelist_file , &
          FILE=file_name_namelist , &
          STATUS = 'OLD' , &
          FORM = 'FORMATTED' , &
          IOSTAT = open_status )
   
   IF ( open_status .NE. 0 ) THEN
      PRINT '(A)' , 'Error opening NAMELIST file.'
      STOP 'namelist_file_open'
   END IF

   !  Undefined values for some fields ares OK.

   sst_to_ice_threshold = -9999
   linear_interpolation = .TRUE.

   !  Read each record from the NAMELIST file.

   READ ( UNIT = namelist_file , NML = record1 )
   READ ( UNIT = namelist_file , NML = record2 )
   READ ( UNIT = namelist_file , NML = record3 )
   READ ( UNIT = namelist_file , NML = record4 )

   PRINT '(A)','Successfully READ the NAMELIST file.'

   IF ( print_echo ) THEN
      WRITE ( UNIT = 6 , NML = record1 )
   END IF

   levels_index = 0
   DO WHILE ( P_levels_in_Pa(levels_index+1) .GT. 0 )
      levels_index = levels_index + 1
! will have crashed already on READ statement above if the following was the case
!      IF levels_index .GE. max_P_levels THEN
!         PRINT '(A,I8)', 'Too many pressure levels, max = ',max_P_levels
!         STOP 'namelist_too_long'
!      ENDIF
   END DO
   number_of_P_levels = levels_index

   levels_index = 0
   DO WHILE ( T_levels(levels_index+1) .GT. -9999999 )
      levels_index = levels_index + 1
   END DO
   T_levels(levels_index+1:number_of_P_levels) = T_levels(levels_index)

   levels_index = 0
   DO WHILE ( NINT(U_levels(levels_index+1)) .NE. end_marker )
      levels_index = levels_index + 1
   END DO
   U_levels(levels_index+1:number_of_P_levels) = U_levels(levels_index)

   levels_index = 0
   DO WHILE ( NINT(V_levels(levels_index+1)) .NE. end_marker )
      levels_index = levels_index + 1
   END DO
   V_levels(levels_index+1:number_of_P_levels) = V_levels(levels_index)

   IF ( print_echo ) THEN
      WRITE ( UNIT = 6 , FMT='(A)' ) ' &RECORD2'
      WRITE ( UNIT = 6 , FMT='(A,F10.2)' ) ' PTOP_IN_PA = ',ptop_in_Pa
      IF ( number_of_P_levels .GT. 0 ) THEN
         WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' P_LEVELS_IN_PA = '
         WRITE ( UNIT = 6 , FMT='(6F10.2)' ) P_levels_in_Pa(1:number_of_P_levels)
      END IF
      IF ( number_of_P_levels .GT. 0 ) THEN
         WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' T_LEVELS = '
         WRITE ( UNIT = 6 , FMT='(6F10.2)' ) T_levels(1:number_of_P_levels)
      END IF
      IF ( number_of_P_levels .GT. 0 ) THEN
         WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' U_LEVELS = '
         WRITE ( UNIT = 6 , FMT='(6F10.2)' ) U_levels(1:number_of_P_levels)
      END IF
      IF ( number_of_P_levels .GT. 0 ) THEN
         WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' V_LEVELS = '
         WRITE ( UNIT = 6 , FMT='(6F10.2)' ) V_levels(1:number_of_P_levels)
      END IF
      WRITE ( UNIT = 6 , FMT='(A,F10.2)' ) ' SST_TO_ICE_THRESHOLD = ',sst_to_ice_threshold
      WRITE ( UNIT = 6 , FMT='(A,L7)' ) ' LINEAR_INTERPOLATION = ',linear_interpolation
      WRITE ( UNIT = 6 , FMT='(A)' ) ' /'
   END IF

   rec3%max_files_constants = 1
   DO WHILE ( constants_full_name(rec3%max_files_constants)(1:20) .NE. '                    ' )
      rec3%max_files_constants = rec3%max_files_constants + 1
   END DO
   rec3%max_files_constants = rec3%max_files_constants - 1
   rec3%max_files_per_single_time = 1
   DO WHILE ( root(rec3%max_files_per_single_time)(1:20) .NE. '                    ' )
      rec3%max_files_per_single_time = rec3%max_files_per_single_time + 1
   END DO
   rec3%max_files_per_single_time = rec3%max_files_per_single_time - 1
   IF ( rec3%max_files_per_single_time .GT. 0 ) THEN
      IF ( print_echo ) THEN
         WRITE ( UNIT = 6 , FMT='(A)' ) ' &RECORD3'
         WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' ROOT = '
         WRITE ( UNIT = 6 , FMT='(A,1X)' ) root(1:rec3%max_files_per_single_time)
         IF ( rec3%max_files_per_single_time .GT. 0 ) THEN
            WRITE ( UNIT = 6 , ADVANCE='NO' , FMT='(A)' ) ' CONSTANTS_FULL_NAME = '
            WRITE ( UNIT = 6 , FMT='(A,1X)' ) constants_full_name(1:rec3%max_files_constants)
         END IF
         WRITE ( UNIT = 6 , FMT='(A,A)' ) ' TERRAIN_FILE = ',TRIM(terrain_file_name)
         WRITE ( UNIT = 6 , FMT='(A)' ) ' /'
      END IF
   ELSE
      PRINT '(A)','You must supply at least a single root name for the first guess file.'
      STOP 'No_name_given_for_first_guess_file'
   END IF
   IF ( ( rec3%max_files_per_single_time .EQ. 0 ) .AND. ( print_echo ) ) THEN
      PRINT '(A)','No constants file found (such as for SST or SNOW), continuing.'
   END IF
   IF ( terrain_file_name(1:20) .EQ. '                    ' ) THEN
      PRINT '(A)','A terrain file name must be supplied in the namelist.'
      STOP 'no_terrain_file_name'
   END IF

   IF ( print_echo ) THEN
      WRITE ( UNIT = 6 , NML = record4 )
   END IF

   !  Assign each local NAMELIST variable to the module name.

   rec1%yyyy_st                   = start_year
   rec1%mm_st                     = start_month
   rec1%dd_st                     = start_day
   rec1%hh_st                     = start_hour
   rec1%min_st                    = start_minute
   rec1%sec_st                    = start_second
   rec1%yyyy_end                  = end_year
   rec1%mm_end                    = end_month
   rec1%dd_end                    = end_day
   rec1%hh_end                    = end_hour
   rec1%min_end                   = end_minute
   rec1%sec_end                   = end_second
   rec1%time_increment_second     = interval
   WRITE (rec1%starting_date,'(I4.4,"-",I2.2,"-",I2.2,"_",I2.2,":",I2.2,":",I2.2)') &
   rec1%yyyy_st , rec1%mm_st , rec1%dd_st , rec1%hh_st , rec1%min_st , rec1%sec_st
   WRITE (rec1%ending_date,'(I4.4,"-",I2.2,"-",I2.2,"_",I2.2,":",I2.2,":",I2.2)') &
   rec1%yyyy_end , rec1%mm_end , rec1%dd_end , rec1%hh_end , rec1%min_end , rec1%sec_end

! should code w/o the duplicate info in rec2%number_of_{P,new}_levels
   rec2%ptop                      = ptop_in_Pa
   rec2%number_of_new_levels      = number_of_P_levels
   rec2%number_of_P_levels        = number_of_P_levels
   rec2%new_levels                = P_levels_in_Pa
   rec2%P_levels                  = P_levels_in_Pa
   rec2%T_levels                 =  T_levels
   rec2%U_levels                  = U_levels
   rec2%V_levels                  = V_levels
   rec2%humid		          = humid
   rec2%sst_to_ice_threshold      = sst_to_ice_threshold
   rec2%linear_interpolation      = linear_interpolation

   rec3%root                      = root
   rec3%constants_full_name       = constants_full_name
   rec3%terrain_file_name         = terrain_file_name
   
   rec4%print_echo                = print_echo 
   rec4%print_debug               = print_debug 
   rec4%print_mask                = print_mask 
   rec4%print_interp              = print_interp 
   rec4%print_link_list_store     = print_link_list_store 
   rec4%print_array_store         = print_array_store 
   rec4%print_header              = print_header 
   rec4%print_output              = print_output 
   rec4%print_file                = print_file 
   rec4%print_f77_info            = print_f77_info

   !  We are finished with the NAMELIST file.

   CLOSE (UNIT=namelist_file)

   IF ( rec4%print_file ) THEN
      PRINT '(A,A/)' , 'CLOSE file: ',file_name_namelist
   END IF

   !  There are a few checks we can perfom on the input data.

   !  Did the user use a 4-digit year, it is required ya know.

   IF ( ( rec1%starting_date(1:2) .LT. '19' ) .OR. &
        ( rec1%starting_date(1:2) .GT. '20' ) .OR. &
        ( rec1%ending_date(1:2)   .LT. '19' ) .OR. &
        ( rec1%ending_date(1:2)   .GT. '20' ) ) THEN

      PRINT '(A)','The namelist dates require a 4 digit year that starts' 
      PRINT '(A)','with either "19" or "20".'
      STOP '4_digit_year'
   END IF

   !  Is the ending time before the starting time?

   IF ( rec1%ending_date .LT. rec1%starting_date ) THEN
      PRINT '(A,A,A,A,A)','The ending date ',rec1%ending_date,' is prior to the starting date ',rec1%starting_date,'.'
      STOP 'prior_ending_date'
   END IF

   !  Are any of the new pressure levels duplicated?

   DO i = 1 , rec2%number_of_P_levels - 1
      DO j = i+1 , rec2%number_of_P_levels 
         IF ( ABS ( rec2%P_levels(i) - rec2%P_levels(j) ) .LT. 1 ) THEN
            PRINT '(A)','Found two identical pressure levels in P_levels_in_Pa.'
            PRINT '(A,F10.1,A)','Remove one of the values of ',rec2%P_levels(i),'.'
            STOP 'duplicate_levels_in_namelist'
         END IF
      END DO
   END DO

   !  The input pressure arrays need to be in bottom-up order.

   DO i = 1 , rec2%number_of_P_levels - 1
      IF ( rec2%P_levels(i) .LT. rec2%P_levels(i+1) ) THEN
            PRINT '(A)','The pressure levels in the P_levels_in_Pa array must be ordered bottom up.'
            PRINT '(A,F10.1,A,F10.1,A)','The pressure ',rec2%P_levels(i),' is less than ',rec2%P_levels(i+1),'.'
            STOP 'out_of_order_levels_in_namelist'
      END IF
   END DO

   !  Print out a nice message letting folks know which particular time
   !  periods we plan to process in this program.  The current date is
   !  set to the requested starting_date, then it is incremented by the
   !  time interval specified in the namelist.  Once the incremented date
   !  is past the requested ending_date, we know we have reached the end
   !  of the number of time periods to process.  

   current_date = rec1%starting_date
   
   rec1%total_times = 0
   time_loop : DO WHILE ( current_date .LE. rec1%ending_date )

      rec1%total_times = rec1%total_times + 1
      IF ( rec4%print_echo ) THEN
         PRINT '(A,A)','Time to process:              ',current_date
      END IF

      CALL geth_newdate ( next_date , current_date , rec1%time_increment_second ) 

      current_date = next_date

   END DO time_loop

   IF ( rec4%print_echo ) THEN
      PRINT '(A,I8)','The total number of times periods to process (as given in the NAMELIST) is ',&
      rec1%total_times
   END IF

END SUBROUTINE proc_namelist






